@@ -2,18 +2,27 @@ package MooseX::Declare::Syntax::RoleApplication;
 
 use Moose::Role;
 
+use aliased 'MooseX::Declare::Context::Namespaced';
+
 use namespace::clean -except => 'meta';
 
 with qw(
     MooseX::Declare::Syntax::OptionHandling
 );
 
+around context_traits => sub { shift->(@_), Namespaced };
+
 sub add_with_option_customizations {
     my ($self, $ctx, $package, $roles) = @_;
 
     # consume roles
-    $ctx->add_scope_code_parts(
-        sprintf 'with %s', join ', ', map { "'$_'" } @{ $roles },
+    $ctx->add_early_cleanup_code_parts(
+        sprintf 'Moose::Util::apply_all_roles(%s->meta, %s)',
+            $package,
+            join ', ',
+            map  { "q[$_]" }
+            map  { $ctx->qualify_namespace($_) }
+                @{ $roles },
     );
 
     return 1;
